#![cfg_attr(
    all(not(debug_assertions), target_os = "windows"),
    windows_subsystem = "windows"
)]

mod led_color;
mod screen;
mod screen_color_picker;

use led_color::LedColor;
use paris::*;
use screen_color_picker::ScreenColorPicker;
use std::{
    sync::{Arc, Mutex},
    time::Instant,
};

#[tauri::command]
fn refresh_displays() {
    match ScreenColorPicker::global().refresh_displays() {
        Ok(_) => {}
        Err(error) => {
            error!("{}", error)
        }
    }
}

#[tauri::command]
async fn take_snapshot() -> Arc<Mutex<Vec<String>>> {
    let start = Instant::now();

    let screenshot_futures = match ScreenColorPicker::global().take_screenshots_for_all() {
        Ok(bitmaps) => {
            info!("bitmaps len: {}", bitmaps.len());
            match ScreenColorPicker::global().screens.lock() {
                Ok(screens) => Some(Vec::from_iter(screens.iter().enumerate().map(
                    |(index, screen)| {
                        bitmap_to_webp_base64(screen.width, screen.height, bitmaps[index].to_vec())
                    },
                ))),
                Err(error) => {
                    error!("can not lock screens. {}", error);
                    None
                }
            }
        }
        Err(error) => {
            error!("can not take screenshots for all. {}", error);
            None
        }
    };
    match screenshot_futures {
        Some(futures) => {
            let mut handles = Vec::with_capacity(futures.len());

            for fut in futures {
                handles.push(tokio::spawn(fut));
            }

            let mut results = Vec::with_capacity(handles.len());
            for handle in handles {
                results.push(handle.await.unwrap());
            }
            println!("运行耗时: {:?}", start.elapsed());
            Arc::new(Mutex::new(results))
        }
        None => Arc::new(Mutex::new(Vec::new())),
    }
}

#[tauri::command]
async fn get_led_strip_colors() -> Result<Vec<LedColor>, String> {
    let screens = ScreenColorPicker::global()
        .screens
        .lock()
        .map_err(|error| {
            error!("can not lock ScreenColorPick.screens. {}", error);
            "failed to lock."
        })?;
    let mut colors = Vec::new();
    for screen in screens.iter() {
        let result = screen.get_top_colors();
        if let Ok(result) = result {
            colors.extend_from_slice(&result);
        } else if let Err(result) = result {
            return Err(format!("can not get led strip colors. {}", result));
        }
    }
    Ok(colors)
}

async fn bitmap_to_webp_base64(width: usize, height: usize, bitmap: Vec<u8>) -> String {
    let mut bitflipped = Vec::with_capacity(width * height * 3);
    let stride = bitmap.len() / height;

    // let mut img = Image::new(w as u32, h as u32);
    for y in 0..height {
        for x in 0..width {
            let i = stride * y + 4 * x;
            bitflipped.extend_from_slice(&[bitmap[i + 2], bitmap[i + 1], bitmap[i]]);
            // img.set_pixel(
            //     x as u32,
            //     y as u32,
            //     Pixel::new(buffer[i + 2], buffer[i + 1], buffer[i]),
            // );
        }
    }

    let webp_memory =
        webp::Encoder::from_rgb(bitflipped.as_slice(), width as u32, height as u32).encode(100.0);
    return base64::encode(&*webp_memory);
}

fn main() {
    tauri::Builder::default()
        .invoke_handler(tauri::generate_handler![
            take_snapshot,
            refresh_displays,
            get_led_strip_colors
        ])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
