use std::sync::{Arc, Mutex};

use anyhow::Ok;
use scrap::Capturer;

use crate::led_color::LedColor;

pub struct Screen {
    bitmap: Arc<Mutex<Option<Vec<u8>>>>,
    capturer: Option<Capturer>,
    init_error: Option<anyhow::Error>,
    pub width: usize,
    pub height: usize,
    pub led_number_of_x: usize,
    pub led_number_of_y: usize,
}

impl Screen {
    pub fn new(capturer: Capturer, width: usize, height: usize) -> Self {
        Self {
            bitmap: Arc::new(Mutex::new(None)),
            capturer: Some(capturer),
            init_error: None,
            width,
            height,
            led_number_of_x: 0,
            led_number_of_y: 0,
        }
    }

    pub fn new_failed(init_error: anyhow::Error, width: usize, height: usize) -> Self {
        Self {
            bitmap: Arc::new(Mutex::new(None)),
            capturer: None,
            init_error: Some(init_error),
            width,
            height,
            led_number_of_x: 0,
            led_number_of_y: 0,
        }
    }

    pub fn set_number_of_leds(&mut self, led_number_of_x: usize, led_number_of_y: usize) -> &Self {
        self.led_number_of_x = led_number_of_x;
        self.led_number_of_y = led_number_of_y;
        self
    }

    pub fn take(&mut self) -> anyhow::Result<Vec<u8>> {
        match self.capturer.as_mut() {
            Some(capturer) => {
                let buffer = capturer
                    .frame()
                    .map_err(|error| anyhow::anyhow!("failed to frame of display. {}", error))?;

                self.bitmap = Arc::new(Mutex::new(Some(buffer.to_vec())));
                anyhow::Ok(buffer.to_vec())
            }
            None => anyhow::bail!("Do not initialized"),
        }
    }

    pub fn get_top_colors(&self) -> anyhow::Result<Vec<LedColor>> {
        self.get_x_colors(XPosition::Top)
    }
    pub fn get_bottom_colors(&self) -> anyhow::Result<Vec<LedColor>> {
        self.get_x_colors(XPosition::Bottom)
    }

    fn get_x_colors(&self, position: XPosition) -> anyhow::Result<Vec<LedColor>> {
        if self.led_number_of_x == 0 {
            return Ok(vec![]);
        }

        let bitmap = self
            .bitmap
            .lock()
            .map_err(|error| anyhow::anyhow!("can not lock Screen#bitmap. {}", error))?;
        match bitmap.as_ref() {
            Some(bitmap) => {
                let cell_size_x = self.width / self.led_number_of_x;
                let cell_size_y = self.height / 5;
                let cell_size = cell_size_x * cell_size_y;
                let y_range = match position {
                    XPosition::Top => 0..cell_size_y,
                    XPosition::Bottom => self.height - cell_size_y..self.height,
                };

                let mut colors = vec![LedColor::default(); self.led_number_of_x];
                let stride = bitmap.len() / self.height;

                for pos in 0..self.led_number_of_x {
                    let mut y_range = y_range.to_owned();

                    let mut r = 0u32;
                    let mut g = 0u32;
                    let mut b = 0u32;
                    for x in pos * cell_size_x..(pos + 1) * cell_size_x {
                        for y in y_range.to_owned() {
                            let i = stride * y + 4 * x;
                            r += bitmap[i + 2] as u32;
                            g += bitmap[i + 1] as u32;
                            b += bitmap[i] as u32;
                        }
                    }
                    colors[pos] = LedColor::new(
                        (r / cell_size as u32) as u8,
                        (g / cell_size as u32) as u8,
                        (b / cell_size as u32) as u8,
                    );
                }
                return Ok(colors);
            }
            None => Ok(vec![]),
        }
    }
}

unsafe impl Send for Screen {}

enum XPosition {
    Top,
    Bottom,
}
